package com.nisum.offertransactionservice.controller;

import com.github.tomakehurst.wiremock.junit.WireMockRule;
import com.nisum.offertransactionservice.dto.Item;
import com.nisum.offertransactionservice.dto.OfferTransactionRequest;
import com.nisum.offertransactionservice.dto.OfferTransactionResponse;
import org.junit.Before;
import org.junit.Rule;
import org.junit.jupiter.api.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.cloud.contract.wiremock.AutoConfigureWireMock;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpMethod;
import org.springframework.http.RequestEntity;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.junit4.SpringRunner;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import static com.github.tomakehurst.wiremock.client.WireMock.*;



@RunWith(SpringRunner.class)
@SpringBootTest(webEnvironment= SpringBootTest.WebEnvironment.DEFINED_PORT)
@AutoConfigureWireMock
public class OfferTransactionControllerIntegrationTest extends AbstractTestBase{

    @Rule
    public WireMockRule wireMockRule = new WireMockRule(7072);

    @Autowired
    private TestRestTemplate testRestTemplate;


    @Before
    public void setUp() {
        mockForOfferTransactionCall();
    }

    private void mockForOfferTransactionCall() {
        mWireMockServer.stubFor(post(urlEqualTo("/offerTransactionCall"))
                .willReturn(aResponse()
                        .withStatus(200)
                        .withHeader("Content-Type", "application/json")
                        .withBodyFile("response.json")
                ));
    }

    @Test
    public void getOfferTransactionResponseTest() {
        URI uri = URI.create("http://localhost:7072/offerTransactionCall");
        RequestEntity requestEntity = new RequestEntity(getOfferTransactionRequest(),HttpMethod.POST,uri);

        ResponseEntity<OfferTransactionResponse> responseEntity = testRestTemplate.exchange("http://localhost:7072/offerTransactionCall", HttpMethod.POST,requestEntity,
                new ParameterizedTypeReference<OfferTransactionResponse>(){}
        );
        OfferTransactionResponse offerTransactionResponse = responseEntity.getBody();
        System.out.println(offerTransactionResponse);

    }

    private OfferTransactionRequest getOfferTransactionRequest() {
        OfferTransactionRequest offerTransactionRequest = new OfferTransactionRequest();
        offerTransactionRequest.setHhId(54321L);
        offerTransactionRequest.setItemList(getItemList());
        return offerTransactionRequest;
    }

    private List<Item> getItemList() {
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 13.05);
        Item item2 = new Item("Thumpsup", "11", 14.05);
        itemList.add(item1);
        itemList.add(item2);
        return itemList;
    }
}
